from django.db import models
from django.conf import settings
from django.contrib.auth.models import User

class MediaFile(models.Model):
    MEDIA_TYPE_CHOICES = [
        ('image', 'Image'),
        ('video', 'Video'),
        ('file', 'File'),
        ('voice', 'Voice'),
    ]

    file = models.FileField(upload_to='uploads/media_files/')
    media_type = models.CharField(max_length=10, choices=MEDIA_TYPE_CHOICES)
    uploaded_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f"{self.media_type} - {self.file.name}"

class PersonalChat(models.Model):
    sender = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name='sent_personal_messages')
    receiver = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name='received_personal_messages')
    content = models.TextField(null=True, blank=True)
    media_files = models.ManyToManyField(MediaFile, related_name='personal_chats', blank=True)
    is_read = models.BooleanField(default=False)
    timestamp = models.DateTimeField(auto_now_add=True)
    deleted_for = models.ManyToManyField(settings.AUTH_USER_MODEL, related_name='deleted_personal_chats', blank=True)

    def __str__(self):
        return f"Chat from {self.sender} to {self.receiver} at {self.timestamp}"
    
class GroupChat(models.Model):
    name = models.CharField(max_length=255)
    description = models.TextField(null=True, blank=True)
    admin = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name='administered_groups')
    members = models.ManyToManyField(settings.AUTH_USER_MODEL, related_name='group_chats')
    created_at = models.DateTimeField(auto_now_add=True)
    agora_group_id = models.CharField(max_length=100, null=True, blank=True)
    profile_image = models.ImageField(upload_to="group_profile_image/", null=True, blank=True)

    def __str__(self):
        return self.name
    
class GroupMessage(models.Model):
    group = models.ForeignKey(GroupChat, on_delete=models.CASCADE, related_name='messages')
    sender = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    content = models.TextField(null=True, blank=True)
    media_files = models.ManyToManyField(MediaFile, related_name='group_messages', blank=True)
    timestamp = models.DateTimeField(auto_now_add=True)
    deleted_for = models.ManyToManyField(settings.AUTH_USER_MODEL, related_name='deleted_group_messages', blank=True)


    def __str__(self):
        return f"Message in {self.group.name} by {self.sender} at {self.timestamp}"
    



class ActiveStatus(models.Model):
    STATUS_CHOICES = [
        (1, 'Online'),
        (2, 'Inactive'),
        (3, 'Offline'),
    ]

    user = models.OneToOneField(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name='active_status_chat_app')
    status = models.IntegerField(choices=STATUS_CHOICES, default=3)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return f"{self.user.username} - {self.get_status_display()}"


class ChatRoom(models.Model):
    conversation_id = models.CharField(max_length=100, unique=True)
    name = models.CharField(max_length=255, blank=True)
    description = models.TextField(null=True, blank=True)
    owner = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name="owned_chatrooms")
    members = models.ManyToManyField(settings.AUTH_USER_MODEL, related_name="chatrooms")
    max_users = models.IntegerField(default=300)
    created_at = models.DateTimeField(auto_now_add=True)
    group = models.OneToOneField(GroupChat, on_delete=models.CASCADE, related_name="chatroom", null=True, blank=True)
    deleted_for = models.ManyToManyField(settings.AUTH_USER_MODEL, related_name='deleted_chatrooms', blank=True)

    def __str__(self):
        return self.name
    
class GroupReport(models.Model):
    reporter = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name="reported_groups")
    group = models.ForeignKey(GroupChat, on_delete=models.CASCADE, related_name="reports")
    # reason = models.TextField()
    created_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f"{self.reporter.username} reported {self.group.name}"
    
class CallSession(models.Model):
    sender = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name="outgoing_calls")
    receiver = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name="incoming_calls")
    channel_name = models.CharField(max_length=255, unique=True)
    token = models.TextField()
    uid = models.CharField(max_length=255)
    created_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f"Call between {self.sender.username} and {self.receiver.username}"